#ifndef PROCESSINGSTAGE_H
#define PROCESSINGSTAGE_H

#include "stage.h"
#include "circularbuffertracker.h"
#include "processingconfig.h"
#include "imagedimensions.h"
#include "processingdatatypes.h"


template <typename ELEMENT_TYPE>
class ProcessingStage : public Stage
{
public:
    ProcessingStage (const ProcessingConfig & config, Stage *next = NULL);
    //ProcessingStage (unsigned int bufferCapacity, unsigned int bufferHistory = 0, Stage *next = NULL);
    virtual ~ProcessingStage ();
    virtual std::ptrdiff_t startWrite ();
    virtual void endWrite (std::ptrdiff_t written);
    virtual void notifyEndOfStream ();
    virtual void run ();
    virtual cudaArray* initializeBuffer (unsigned int width, unsigned int height, unsigned int layers);
    virtual cudaArray* initializeBuffer (ImageDimensions const &dim);
private:
    ProcessingStage();

    void setupSpatialGaussians ();
    virtual cudaArray* initializeBuffer ();

    ProcessingConfig prConfig;
    CircularBufferTracker tracker;
    cudaArray *inputImageArray;
    cudaArray *imageDiffArray;
//    cudaArray *probArray;
//    cudaArray *probDiffArray;
//    cudaArray *imageDiffSumArray;
//    cudaArray *probDiffSumArray;
    cudaArray *outputImageArray;
    ImageDimensions imageDim;

//    float     *backgroundSpatialHorizontalGaussian;
//    float     *backgroundSpatialVerticalGaussian;
//    float     *foregroundSpatialHorizontalGaussian;
//    float     *foregroundSpatialVerticalGaussian;

    ProcessingDeviceAux deviceAuxData;

    ProcessingDeviceData<ELEMENT_TYPE> deviceData;
};

#endif // PROCESSINGSTAGE_H
